
#include "serapp.h"
#include <termios.h>
#include <stdio.h>

/* *****************************************************
 * 串口设置由termios结构掌管,成员的值决定串口工作方式
 * 结构成员如下:
 * struct termio
 * {
 * 	unsigned short c_iflag; //输入模式标志
 * 	unsigned short c_oflag; //输出模式标志
 * 	unsigned short c_cflag; //控制模式标志
 * 	unsigned short c_lflag; //local mode flag
 * 	unsigned short c_line;  //line discipline
 * 	unsigned short c_cc[NCC]; //control characters
 * };
 * ****************************************************/



/* *****************************************************
 * 波特率设置函数
 * int fd 	:串口文件描述符
 * int speed	:串口速度
 * return 	:成功 TRUE, 失败 FALSE
 * ****************************************************/
int speed_arr[] = {B115200, B57600, B38400, B19200, B9600, B4800, B2400};
int name_arr[] = {115200, 57600, 38400, 19200, 9600, 4800, 2400};

int set_speed(int fd, int speed)
{
    int i;
    struct termios tio;

    if(tcgetattr(fd, &tio) != TRUE){
	//fprintf(stderr, "Setup speed (tcgetattr) error\n");
	perror("Setup speed (tcgetattr) error");
	return FALSE;
    }

    for(i = 0; i < sizeof(speed_arr) / sizeof(speed_arr[0]); i++){
	if(speed == name_arr[i]){
	    //tcflush(fd, TCIOFLUSH);
	    cfsetispeed(&tio, speed_arr[i]);
	    cfsetospeed(&tio, speed_arr[i]);

	    if(tcsetattr(fd, TCSANOW, &tio) != TRUE){
		//fprintf(stderr, "Setup speed (tcsetattr) error\n");
		perror("Setup speed (tcsetattr) error");
		return FALSE;
	    }
	    tcflush(fd, TCIOFLUSH);

	    return TRUE;
	} 
    }

    fprintf(stderr, "avrboot: Unsupported buadrate\n");
    fprintf(stderr, "avrboot: Must be one of 115200,57600,38400,19200,9600,4800,2400\n");
    return FALSE;
}

/* *****************************************************
 * 校验数据位及停止位设置函数
 * int fd 	:串口文件描述符
 * int databits	:数据位(7 or 8)
 * int stopbits :停止位(1 or 2)
 * int parity	:校验类型(None, Even, Odd, Space)
 * return 	:TURE成功, FALSE失败
 * ****************************************************/
int set_parity(int fd, int databits, int stopbits, int parity)
{
    struct termios tio;

    if(tcgetattr(fd, &tio) != 0){
	//fprintf(stderr, "setup parity (tcgetattr) error\n");
	perror("Setup speed (tcgetattr) error");
	return FALSE;
    }

    tio.c_cflag &= ~CSIZE;

    /* 设置数据位数 */
    switch(databits){ 
	case 7:
	    tio.c_cflag |= CS7;
	    break;
	case 8:
	    tio.c_cflag |= CS8;
	    break;
	default:
	    fprintf(stderr, "Unsupported data size\n");
	    return FALSE;
    }

    /* 设置校验位 */
    switch(parity){
	case 'n':
	case 'N':
	    tio.c_cflag &= ~PARENB;/* Clear parity enable */
	    tio.c_iflag &= ~INPCK; /* Enable parity checking */
	    break;
	case 'o':
	case 'O':
	    tio.c_cflag |= (PARODD | PARENB); /* odd */
	    tio.c_iflag |= INPCK; /* Disable parity checking */
	    break;
	case 'e':
	case 'E':
	    tio.c_cflag |= PARENB; /* Enable parity */
	    tio.c_cflag &= ~PARODD; /* even */
	    tio.c_iflag |= INPCK; /* Disable parity checking */
	    break;
	case 's':
	case 'S': /* as no parity */
	    tio.c_cflag &= ~PARENB;
	    tio.c_cflag &= ~CSTOPB;
	    break;
	default:
	    fprintf(stderr, "Unsupported parity\n");
	    return FALSE;
    }

    /* set input parity option */
    if(parity != 'n')
	tio.c_iflag |= INPCK;

    /* 设置停止位 */
    switch(stopbits){
	case 1:
	    tio.c_cflag &= ~CSTOPB;
	    break;
	case 2:
	    tio.c_cflag |= CSTOPB;
	default:
	    fprintf(stderr, "Unsupported stop bits\n");
	    return FALSE;
    }

    /* 设置RAW模式通信 */ /* 软件控制数据流方式 */
    tio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); /* input */
    tio.c_oflag &= ~OPOST; /* output */

    tio.c_cc[VTIME] = 10; /* (10) timer out 1 seconds */
    tio.c_cc[VMIN] = 0; /* Update the options and do it NOW */
    //tio.c_cc[VMIN] = 1; /* Block reading until 1 char received */

    /* 设置完成将数据写入 */
    tcflush(fd, TCIFLUSH);
    if(tcsetattr(fd, TCSANOW, &tio) != TRUE){
	//fprintf(stderr, "setup parity (tcsetattr) error\n");
	perror("Setup speed (tcsetattr) error");
	return FALSE;
    }

    return TRUE;
}


/* end of file */
